from dataclasses import dataclass, field
# from collections import namedtuple
from dashboards.influx_queries import Query
from grafanalib.core import TimeSeries, Text, Stat, Template, Repeat, Threshold
from dashboards.dashboard_base import get_influx_target
from dashboards.legends import Units
from numbers import Number

# PanelInfos = namedtuple("PanelInfos", ("name", "unit"))


@dataclass
class PanelInfos:
    name: str
    unit: Units
    absthreshold: Number = field(default=None)


def is_regex(name):
    return name[0] == "/" and name[-1] == "/"


def get_time_series_panel(panel_infos: PanelInfos,
                          data_source: str,
                          measurment_name: str,
                          *,
                          where=None,
                          group_by=None,
                          overrides=None,
                          **kwargs):
    query = Query(select_=panel_infos.name,
                  from_=measurment_name,
                  where_=where,
                  group_by=group_by,
                  from_string=not is_regex(measurment_name),
                  select_string=not is_regex(panel_infos.name))
    new_kwargs = {**kwargs}
    if panel_infos.absthreshold is not None:
        new_kwargs.update({'thresholdType': 'absolute',
                           'thresholds': [Threshold('green', 0, 0.0),
                                          Threshold('red', index=1, value=float(panel_infos.absthreshold), op='lt'), ],
                           'thresholdsStyleMode': 'line',
                           }
                          )

    return TimeSeries(
        title=panel_infos.name,
        dataSource=data_source,
        targets=[get_influx_target(str(query))],
        unit=panel_infos.unit,
        pointSize=9,
        overrides=overrides,
        **new_kwargs,

    )


def get_text_panel(content: str, *, mode='markdown', **kwargs) -> Text:
    return Text(
        content=content,
        mode=mode,
        **kwargs
    )


def get_stat_panel(title: str,
                   dataSource: str,
                   stat_query: Query,
                   repeat: Template = None,
                   alias: str = "",
                   *,
                   maxPerRow=0,
                   **kwargs):
    new_kwargs = {
        'alignment': 'center',
        'colorMode': 'value',
        'graphMode': 'area',
        'reduceCalc': 'last',
        'orientation': 'auto',
        'transparent': True,
    }
    new_kwargs.update(kwargs)
    if repeat:
        rep_args = ['h', repeat.name]
        if maxPerRow:
            rep_args.append(maxPerRow)
        new_kwargs.setdefault('repeat', Repeat(*rep_args))
    return Stat(
        title=title,
        dataSource=dataSource,
        targets=[get_influx_target(str(stat_query), alias=alias)],
        thresholdType='percentage',
        thresholds=[Threshold('green', 0, 0.0), Threshold('yellow', 1, 50.0), Threshold('red', 2, 80.0)],
        ** new_kwargs,
    )
