import pandas as pd
import pytest

from cbutil.ncu_parser import extract_raw_counter


@pytest.fixture
def sample_data():
    # Create sample data for testing
    data = {
        ("Block Size", ""): [128, 256, 512],
        ("Grid Size", ""): [64, 128, 256],
        ("device__attribute_display_name", ""): ["GPU1", "GPU2", "GPU3"],
        ("dram__bytes_write.sum", "Bytes"): [1e9, 2e9, 3e9],
        ("dram__bytes_read.sum", "GBytes"): [0.5, 1., 1.5],
        ("dram__bytes_write.sum.per_second", "MByte/s"): [100, 200, 300],
        ("dram__bytes_read.sum.per_second", "MByte/s"): [50, 100, 150],
        ("dram__bytes.sum.per_second", "MByte/s"): [150, 300, 450],
        ("gpu__time_duration.sum", "msecond"): [10, 20, 30],
        ("smsp__cycles_elapsed.avg.per_second", "Cycles/s"): [1000, 2000, 3000],
        ("smsp__sass_thread_inst_executed_op_dfma_pred_on.sum.per_cycle_elapsed", ""): [500, 1000, 1500],
        ("smsp__sass_thread_inst_executed_op_dadd_pred_on.sum.per_cycle_elapsed", ""): [0, 0, 0],
        ("smsp__sass_thread_inst_executed_op_dmul_pred_on.sum.per_cycle_elapsed", ""): [0, 0, 0],
    }
    return pd.DataFrame(data)


def test_extract_raw_counter(sample_data):
    fields, tags = extract_raw_counter(sample_data)

    # Check fields DataFrame
    assert list(fields.columns) == [
        "Memory write data volume [GBytes]",
        "Memory read data volume [GBytes]",
        "Memory data volume [GBytes]",
        "Memory write bandwidth [MByte/s]",
        "Memory read bandwidth [MByte/s]",
        "Memory bandwidth [MByte/s]",
        "Runtime [s]",
        "SMSP Cycles [Cycles/s]",
        "SMSP Cycles",
        "FP inst per cycle",
        "Total FP inst",
        "Operational intensity",
        "P_max [MFlop/s]",
        "DP [MFlop/s]",
    ]
    assert fields.shape == (3, 14)

    # Check values in fields DataFrame
    assert fields["Memory write data volume [GBytes]"].tolist() == [1.0, 2.0, 3.0]
    assert fields["Memory read data volume [GBytes]"].tolist() == [0.5, 1.0, 1.5]
    assert fields["Memory data volume [GBytes]"].tolist() == [1.5, 3.0, 4.5]
    # Add more checks for other columns in the fields DataFrame

    # Check tags DataFrame
    assert list(tags.columns) == ["Block Size", "Grid Size", "GPU"]
    assert tags.shape == (3, 3)

    # Check values in tags DataFrame
    assert tags["Block Size"].tolist() == [128, 256, 512]
    assert tags["Grid Size"].tolist() == [64, 128, 256]
    assert tags["GPU"].tolist() == ["GPU1", "GPU2", "GPU3"]
